"""Config storage for Anergy Assistant."""

import logging
import uuid
from pathlib import Path
from typing import Any, Final

import yaml
from anyio import open_file

from ..constants import ROOT_LOGGER_NAME

LOGGER = logging.getLogger(ROOT_LOGGER_NAME)

CONFIG_DATA_FILE: Final = "config_data.yaml"


class ConfigFileError(Exception):
    """Config file errors."""


class DeviceNotFoundError(Exception):
    """The device was not found in the configuration."""


class DeviceConfigMissingParameterError(Exception):
    """Device configuration exception."""

    def __init__(self, missing_param: str) -> None:
        """Create a DeviceConfigError instance."""
        super().__init__(f"Parameter {missing_param} is missing in the configuration")


def merge_dicts(source: dict, destination: dict) -> dict:
    """Merge 2 dictionaries."""
    for key, value in source.items():
        if isinstance(value, dict):
            # get node or create one
            node = destination.setdefault(key, {})
            merge_dicts(value, node)
        else:
            destination[key] = value

    return destination


def get_dict_value(d: dict, key: str, default_value: Any = None) -> Any:
    """Get the value of a dictionary."""
    parent = d
    subkeys = key.split("/")
    for index, subkey in enumerate(subkeys):
        if index == (len(subkeys) - 1):
            value = parent.get(subkey, default_value)
            if value is None:
                # replace None with default
                return default_value
            return value
        if subkey not in parent:
            # requesting subkey from a non existing parent
            return default_value
        parent = parent[subkey]
    return default_value


def set_dict_value(d: dict, key: str, value: Any) -> None:
    """Set a value in a dictionary."""
    parent = d
    subkeys = key.split("/")
    for index, subkey in enumerate(subkeys):
        if index == (len(subkeys) - 1):
            parent[subkey] = value
        else:
            parent.setdefault(subkey, {})
            parent = parent[subkey]


class ConfigSectionStorageBase:
    """Base class for config file storage."""

    def __init__(self, data_folder: Path, section: str) -> None:
        """Create a ConfigStorage instance."""
        self._data_folder = data_folder
        self._section = section

    @property
    def _config_data_file(self) -> Path:
        return self._data_folder / f"{self._section}_{CONFIG_DATA_FILE}"

    def delete_config_file(self) -> None:
        """Delete the config file on the disk."""
        if self._config_data_file.exists():
            self._config_data_file.unlink()


class ConfigSectionStorage(ConfigSectionStorageBase):
    """Load and store a section from a config file."""

    def __init__(self, data_folder: Path, section: str) -> None:
        """Create a ConfigStorage instance."""
        super().__init__(data_folder, section)
        self._config: dict = {}
        self._data: dict = {}
        self._merged_data: dict = {}

    async def initialize(self, config_file: Path) -> None:
        """Load the config file data."""
        try:
            async with await open_file(config_file) as stream:
                LOGGER.debug("Successfully opened config file %s", config_file)
                config = yaml.safe_load(await stream.read())
                self._config = config.get(self._section, {})
                LOGGER.debug("config file %s successfully loaded", config_file)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to parse the config file")
            raise ConfigFileError from error
        except FileNotFoundError as error:
            LOGGER.exception("File %s not found", str(config_file))
            raise ConfigFileError from error
        try:
            async with await open_file(self._config_data_file) as stream:
                LOGGER.debug("Successfully opened config data file %s", config_file)
                self._data = yaml.safe_load(await stream.read())
                LOGGER.debug("config file %s successfully loaded", config_file)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to parse the config file")
            raise ConfigFileError from error
        except FileNotFoundError:
            self._data = {}
        self._merge_data()

    def _merge_data(self) -> None:
        """Merge the data dictionaries."""
        self._merged_data = merge_dicts(self._data, self._config.copy())

    def store(self) -> None:
        """Store the config data."""
        try:
            #            async with await open_file(self._data_folder/CONFIG_DATA_FILE, mode='w') as stream:
            self._data_folder.mkdir(parents=True, exist_ok=True)
            with self._config_data_file.open(mode="w") as stream:
                yaml.safe_dump(self._data, stream)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to write the config data file")
            raise ConfigFileError from error

    def set(self, key: str, value: Any) -> None:
        """Set value(s) for a specific key/path in persistent storage."""
        # we support a multi level hierarchy by providing the key as path,
        # with a slash (/) as splitter.
        set_dict_value(self._data, key, value)
        self._merge_data()
        self.store()

    def get(self, key: str, default: Any = None) -> Any:
        """Get value(s) for a specific key/path in persistent storage."""
        return get_dict_value(self._merged_data, key, default)

    def get_param(self, key: str, default: Any = None) -> Any:
        """Get value(s) for a specific key/path in persistent storage."""
        result = get_dict_value(self._merged_data, key, default)
        if result is None:
            raise DeviceConfigMissingParameterError(key)
        return result

    def as_dict(self) -> dict:
        """Get the configuration data as a dictionary."""
        return self._merged_data


class DeviceConfigStorage(ConfigSectionStorageBase):
    """Load and store the device configuration from a config file."""

    def __init__(self, data_folder: Path) -> None:
        """Create a device config storage instance."""
        super().__init__(data_folder, "devices")
        self._config: list[dict] = []
        self._data: list[dict] = []
        self._merged_data: list[dict] = []

    async def initialize(self, config_file: Path) -> None:
        """Load the config file data."""
        try:
            async with await open_file(config_file) as stream:
                LOGGER.debug("Successfully opened config file %s", config_file)
                config = yaml.safe_load(await stream.read())
                self._config = config.get(self._section, [])
                LOGGER.debug("config file %s successfully loaded", config_file)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to parse the config file")
            raise ConfigFileError from error
        except FileNotFoundError as error:
            LOGGER.exception("File %s not found", str(config_file))
            raise ConfigFileError from error
        try:
            async with await open_file(self._config_data_file) as stream:
                LOGGER.debug("Successfully opened config data file %s", config_file)
                self._data = yaml.safe_load(await stream.read())
                LOGGER.debug("config file %s successfully loaded", config_file)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to parse the config file")
            raise ConfigFileError from error
        except FileNotFoundError:
            self._data = []
        self._merge_data()

    def _find_device_in_data(self, device_id: str) -> dict:
        for device in self._data:
            if device.get("id") == device_id:
                return device
        return {}

    def _merge_device(self, device: dict) -> dict:
        device_id = device.get("id")
        if device_id is not None:
            data = self._find_device_in_data(device_id)
            return merge_dicts(data, device.copy())
        return {}

    def _merge_data(self) -> None:
        """Merge the data dictionaries."""
        # self._merged_data = merge_dicts(self._data, self._config.copy())
        self._merged_data = [self._merge_device(device) for device in self._config]

    def store(self) -> None:
        """Store the config data."""
        try:
            #            async with await open_file(self._data_folder/CONFIG_DATA_FILE, mode='w') as stream:
            self._data_folder.mkdir(parents=True, exist_ok=True)
            with self._config_data_file.open(mode="w") as stream:
                yaml.safe_dump(self._data, stream)
        except yaml.YAMLError as error:
            LOGGER.exception("Failed to write the config data file")
            raise ConfigFileError from error

    def has_device_config(self, device_id: uuid.UUID) -> bool:
        """Check if a configuration for a device is available."""
        return any(device.get("id") == str(device_id) for device in self._config)

    def get_device_config(self, device_id: uuid.UUID) -> dict:
        """Get the configuration of a device."""
        if self._merged_data is not None:
            for device in self._merged_data:
                if device.get("id") == str(device_id):
                    return device
        raise DeviceNotFoundError

    def get(self, device_id: uuid.UUID, key: str) -> Any:
        """Get a value of a device configuration."""
        device = self.get_device_config(device_id)
        return get_dict_value(device, key)

    def set(self, device_id: uuid.UUID, key: str, value: Any) -> None:
        """Set a config parameter of a device."""
        if not self.has_device_config(device_id):
            raise DeviceNotFoundError

        for device in self._data:
            if device.get("id") == str(device_id):
                set_dict_value(device, key, value)
                self._merge_data()
                self.store()
                return
        device = {"id": str(device_id)}
        set_dict_value(device, key, value)
        self._data.append(device)

        self._merge_data()
        self.store()

    def as_list(self) -> list:
        """Get the configuration data as a list."""
        return self._merged_data


class ConfigStorage:
    """Configuration file storage."""

    def __init__(self, data_folder: Path) -> None:
        """Create an instance of the ConfigurationStorage."""
        self._mqtt = ConfigSectionStorage(data_folder, "mqtt")
        self._homeassistant = ConfigSectionStorage(data_folder, "homeassistant")
        self._home = ConfigSectionStorage(data_folder, "home")
        self._devices = DeviceConfigStorage(data_folder)
        self._emhass = ConfigSectionStorage(data_folder, "emhass")

    async def initialize(self, config_file: Path) -> None:
        """Load the config file data."""
        await self._mqtt.initialize(config_file)
        await self._homeassistant.initialize(config_file)
        await self._home.initialize(config_file)
        await self._devices.initialize(config_file)
        await self._emhass.initialize(config_file)

    @property
    def mqtt(self) -> ConfigSectionStorage:
        """Return the configuration storage for mqtt."""
        return self._mqtt

    @property
    def homeassistant(self) -> ConfigSectionStorage:
        """Return the configuration storage for homeassistant."""
        return self._homeassistant

    @property
    def home(self) -> ConfigSectionStorage:
        """Return the configuration storage for home."""
        return self._home

    @property
    def devices(self) -> DeviceConfigStorage:
        """Return the configuration storage for devices."""
        return self._devices

    @property
    def emhass(self) -> ConfigSectionStorage:
        """Return the configuration storage for emhss."""
        return self._emhass

    def delete_config_file(self) -> None:
        """Delete the config file on the disk."""
        self._mqtt.delete_config_file()
        self._homeassistant.delete_config_file()
        self._home.delete_config_file()
        self._devices.delete_config_file()
        self._emhass.delete_config_file()

    def as_dict(self) -> dict:
        """Get the config data dictionary."""
        return {
            "mqtt": self._mqtt.as_dict(),
            "homeassistant": self._homeassistant.as_dict(),
            "home": self._home.as_dict(),
            "devices": self._devices.as_list(),
            "emhass": self._emhass.as_dict(),
        }



"""Views for home measurement API."""

import uuid
from typing import Annotated

from fastapi import APIRouter, Depends, HTTPException, Path, Request

from .schema import ConfigModel, ReadConfigResponse, ReadDeviceConfigResponse
from .use_cases import ReadConfiguration, ReadDeviceConfiguration, WriteDeviceConfiguration

router = APIRouter(prefix="/config")


@router.get("", response_model=ReadConfigResponse)
async def read_configuration(
    request: Request,
    use_case: Annotated[ReadConfiguration, Depends(ReadConfiguration)],
) -> ConfigModel:
    """Rest end point for read all devices."""
    energy_assistant = request.app.energy_assistant if hasattr(request.app, "energy_assistant") else None
    if energy_assistant is None:
        raise HTTPException(status_code=500)
    return await use_case.execute(energy_assistant.config)


@router.get(
    "/device/{device_id}",
    response_model=ReadDeviceConfigResponse,
)
async def read(
    request: Request,
    device_id: Annotated[uuid.UUID, Path(..., description="")],
    use_case: Annotated[ReadDeviceConfiguration, Depends(ReadDeviceConfiguration)],
) -> ConfigModel:
    """REST end point for read a device configuration."""
    energy_assistant = request.app.energy_assistant if hasattr(request.app, "energy_assistant") else None
    if energy_assistant is None:
        raise HTTPException(status_code=500)
    return await use_case.execute(energy_assistant.config, device_id)


@router.put(
    "/device/{device_id}",
    response_model=ReadDeviceConfigResponse,
)
async def write(
    request: Request,
    data: dict,
    device_id: Annotated[uuid.UUID, Path(..., description="")],
    use_case: Annotated[WriteDeviceConfiguration, Depends(WriteDeviceConfiguration)],
) -> ConfigModel:
    """REST end point for read a device configuration."""
    energy_assistant = request.app.energy_assistant if hasattr(request.app, "energy_assistant") else None
    if energy_assistant is None:
        raise HTTPException(status_code=500)
    return await use_case.execute(energy_assistant.config, data, device_id, energy_assistant.home)



"""create device table

Revision ID: efac279d8401
Revises: d43efed24fbc
Create Date: 2023-06-30 15:17:19.222337

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = "efac279d8401"
down_revision = "d43efed24fbc"
branch_labels = None
depends_on = None


def upgrade() -> None:
    op.create_table(
        "devices",
        sa.Column("id", sa.CHAR(32), nullable=False),
        sa.Column("name", sa.VARCHAR(), nullable=False),
        sa.Column("icon", sa.VARCHAR(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )

    with op.batch_alter_table("DeviceMeasurement") as batch_op:
        batch_op.add_column(sa.Column("device_id", sa.CHAR(32), sa.ForeignKey("devices.id")))


def downgrade() -> None:
    with op.batch_alter_table("DeviceMeasurement") as batch_op:
        batch_op.drop_column("device_id")
    op.drop_table("devices")



"""Main api module."""

from fastapi import APIRouter

from .config.views import router as config_router
from .device.views import router as device_router
from .forecast.views import router as forecast_router
from .history.views import router as history_router
from .home_measurement.views import router as home_measurement_router
from .sessionlogs.views import router as session_log_router

router = APIRouter()

router.include_router(device_router)
router.include_router(home_measurement_router)
router.include_router(history_router)
router.include_router(session_log_router)
router.include_router(forecast_router)
router.include_router(config_router)



"""The Device classes."""

from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import StrEnum, auto
from typing import TYPE_CHECKING, Any
from zoneinfo import ZoneInfo

if TYPE_CHECKING:
    import uuid
    from datetime import datetime, tzinfo


@dataclass
class LoadInfo:
    """Information about a deferrable load."""

    device_id: uuid.UUID
    nominal_power: float
    duration: float
    is_continous: bool
    is_constant: bool = False
    start_timestep: int = 0
    end_timestep: int = 0
    is_deferrable: bool = True


@dataclass
class Session:
    """Session data class."""

    id: int
    start: datetime
    start_solar_consumed_energy: float
    start_consumed_energy: float


class SessionStorage(ABC):
    """Session storage base class."""

    @abstractmethod
    async def start_session(
        self,
        device_id: uuid.UUID,
        text: str,
        solar_consumed_energy: float,
        consumed_energy: float,
    ) -> Session:
        """Start a new session."""

    @abstractmethod
    async def update_session(self, id: int, solar_consumed_energy: float, consumed_energy: float) -> None:
        """Update the session with the given id."""

    @abstractmethod
    async def update_session_energy(self, id: int, solar_consumed_energy: float, consumed_energy: float) -> None:
        """Update the session with the given id."""


class Integrator:
    """Integrate a measurement like power to get the energy."""

    def __init__(self) -> None:
        """Initialize the integrator."""
        self.last_measurement: float | None = None
        self.last_timestamp: float | None = None
        self._value: float = 0.0

    @property
    def value(self) -> float:
        """The current value of the integrator."""
        return self._value

    def add_measurement(self, measurement: float, timestamp: float) -> None:
        """Update the value of the integrator with and new measuremenent value."""
        if self.last_measurement is None:
            self.last_measurement = measurement
            self.last_timestamp = timestamp
        else:
            delta_t = timestamp - self.last_timestamp if self.last_timestamp else 0.0
            self.last_timestamp = timestamp
            # print("Delta t: "+ str(delta_t))
            if delta_t > 0.1:
                if measurement > self.last_measurement:
                    self._value = self._value + (
                        delta_t * (self.last_measurement + (measurement - self.last_measurement) / 2)
                    )
                else:
                    self._value = self._value + (delta_t * (measurement + (self.last_measurement - measurement) / 2))

    def restore_state(self, state: float) -> None:
        """Restore the integrator value with a previously saved state."""
        self._value = state


class EnergyIntegrator:
    """Integrates energy based on a real measurement and a self sufficiency value in order to sum up the consumed solar energy."""

    def __init__(self) -> None:
        """Create an energy integrator."""
        self._last_consumed_energy: float = 0.0
        self._consumed_solar_energy: float = 0.0

    @property
    def consumed_solar_energy(self) -> float:
        """The amount of solar energy which has been consumed."""
        return self._consumed_solar_energy

    def add_measurement(self, consumed_energy: float, self_sufficiency: float) -> None:
        """Update the value of the integrator with and new measuremenent value."""
        self._consumed_solar_energy = (
            self._consumed_solar_energy + (consumed_energy - self._last_consumed_energy) * self_sufficiency
        )
        self._last_consumed_energy = consumed_energy

    def restore_state(self, consumed_solar_energy: float, last_consumed_energy: float) -> None:
        """Restores the integrator value with a previously saved state."""
        self._consumed_solar_energy = consumed_solar_energy
        self._last_consumed_energy = last_consumed_energy


class EnergySnapshot:
    """Stores an snapshot of the current energy consumption values."""

    def __init__(self, consumed_solar_energy: float, consumed_energy: float) -> None:
        """Create an energy snapshot."""
        self._consumed_solar_energy = consumed_solar_energy
        self._consumed_energy = consumed_energy

    @property
    def consumed_solar_energy(self) -> float:
        """The amount of consumed solar energy."""
        return self._consumed_solar_energy

    @property
    def consumed_energy(self) -> float:
        """The total amount of consumed energy."""
        return self._consumed_energy


class HomeEnergySnapshot(EnergySnapshot):
    """Stores an snapshot of the current energy consumption values."""

    def __init__(
        self,
        produced_solar_energy: float,
        grid_imported_energy: float,
        grid_exported_energy: float,
    ) -> None:
        """Create an energy snapshot."""
        self._produced_solar_energy = produced_solar_energy
        self._grid_imported_energy = grid_imported_energy
        self._grid_exported_energy = grid_exported_energy

    @property
    def consumed_energy(self) -> float:
        """The total amount of consumed energy."""
        return self.grid_imported_energy - self.grid_exported_energy + self.produced_solar_energy

    @property
    def consumed_solar_energy(self) -> float:
        """Consumed solar energy in kWh."""
        return self.produced_solar_energy - self.grid_exported_energy

    @property
    def produced_solar_energy(self) -> float:
        """The total amount of produced solar energy."""
        return self._produced_solar_energy

    @property
    def grid_imported_energy(self) -> float:
        """The total amount of energy imported from the grid."""
        return self._grid_imported_energy

    @property
    def grid_exported_energy(self) -> float:
        """The total amount of energy exported to the grid."""
        return self._grid_exported_energy


class OnOffState(StrEnum):
    """Representation of a on/off state."""

    ON = "on"
    OFF = "off"
    UNKNOWN = "unknown"

    @classmethod
    def from_str(cls, label: str) -> OnOffState:
        """Convert a string to a OnOffState."""
        label = label.lower()
        if label == "on":
            return cls.ON
        if label == "off":
            return cls.OFF
        return cls.UNKNOWN

    @classmethod
    def from_bool(cls, value: bool) -> OnOffState:
        """Convert a bool to a OnOffState."""
        return cls.ON if value else cls.OFF


class State:
    """Base class for States."""

    def __init__(self, id: str, value: str, attributes: dict | None = None) -> None:
        """Create a state instance."""
        self._id = id
        self._value = value
        self._attributes = attributes if attributes is not None else {}
        self._available = True

    @property
    def id(self) -> str:
        """Id of the state."""
        return self._id

    @property
    def available(self) -> bool:
        """Availability of the state."""
        return self._available

    @property
    def value(self) -> str:
        """State of the state as string."""
        return self._value

    @property
    def numeric_value(self) -> float:
        """Numeric state of the state."""
        try:
            return float(self._value)
        except ValueError:
            return 0.0

    @property
    def attributes(self) -> dict:
        """Attributes of the state."""
        return self._attributes


def assign_if_available(old_state: State | None, new_state: State | None) -> State | None:
    """Return new state in case the state is available, otherwise old state."""
    if new_state and new_state.available:
        return new_state
    return old_state


@dataclass(frozen=True, eq=True)
class StateId:
    """The id of a state."""

    id: str
    channel: str


class StatesRepository(ABC):
    """Abstract base class for a state repositiroy."""

    @abstractmethod
    def get_state(self, id: StateId | str) -> State | None:
        """Get a state from the repository."""

    @abstractmethod
    def get_numeric_states(self) -> dict[str, float]:
        """Get a states from the repository."""

    @abstractmethod
    def get_template_states(self) -> dict:
        """Get a states from the repository."""

    @abstractmethod
    def set_state(self, id: StateId, value: str, attributes: dict | None = None) -> None:
        """Set a state in the repository."""

    @property
    @abstractmethod
    def channel(self) -> str:
        """Get the channel of the State Repository."""

    @abstractmethod
    async def async_read_states(self) -> None:
        """Read the states from the channel asynchronously."""

    @abstractmethod
    async def async_write_states(self) -> None:
        """Send the changed states to hass."""

    @abstractmethod
    def read_states(self) -> None:
        """Read the states from the channel."""

    @abstractmethod
    def write_states(self) -> None:
        """Write the states to the channel."""


class StatesSingleRepository(StatesRepository):
    """Base class for a state repositiroy."""

    def __init__(self, channel: str) -> None:
        """Create a StatesRepository instance."""
        self._channel = channel
        self._read_states: dict[str, State] = dict[str, State]()
        self._write_states: dict[str, State] = dict[str, State]()
        self._template_states: dict[str, dict | Any] | None = None

    def get_state(self, id: StateId | str) -> State | None:
        """Get a state from the repository."""
        if isinstance(id, str):
            return self._read_states.get(id)
        return self._read_states.get(id.id)

    def get_numeric_states(self) -> dict[str, float]:
        """Get a states from the repository."""
        return {k: v.numeric_value for k, v in self._read_states.items()}

    def get_template_states(self) -> dict:
        """Get template states from the repository."""
        if self._template_states is None:
            self._template_states = {}
            states = self.get_numeric_states().items()
            for k, v in states:
                parts = k.split(".")
                if len(parts) > 1:
                    type = parts[0]
                    attribute = parts[1]
                    if type in self._template_states:
                        self._template_states[type][attribute] = v
                    else:
                        self._template_states[type] = {attribute: v}
                else:
                    self._template_states[k] = v
        return self._template_states

    def set_state(self, id: StateId, value: str, attributes: dict | None = None) -> None:
        """Set a state in the repository."""
        self._write_states[id.id] = State(id.id, value, attributes)

    @property
    def channel(self) -> str:
        """Get the channel of the State Repository."""
        return self._channel


class StatesMultipleRepositories(StatesRepository):
    """Base class for a state repositiroy."""

    def __init__(self, repositories: list[StatesRepository]) -> None:
        """Create a StatesRepository instance."""
        self._repositories = repositories

    def get_state(self, id: StateId | str) -> State | None:
        """Get a state from the repository."""
        if isinstance(id, StateId):
            for repository in self._repositories:
                if repository.channel == id.channel:
                    result = repository.get_state(id)
                    if result is not None:
                        return result
        else:
            for repository in self._repositories:
                result = repository.get_state(id)
                if result is not None:
                    return result
        return None

    def get_numeric_states(self) -> dict[str, float]:
        """Get a states from the repository."""
        result: dict[str, float] = {}
        for reposititory in self._repositories:
            result = {**result, **reposititory.get_numeric_states()}
        return result

    def get_template_states(self) -> dict:
        """Get template states from the repository."""
        result: dict = {}
        for reposititory in self._repositories:
            result = {**result, **reposititory.get_template_states()}
        return result

    def set_state(self, id: StateId, value: str, attributes: dict | None = None) -> None:
        """Set a state in the repository."""
        for repository in self._repositories:
            if id.channel == repository.channel:
                repository.set_state(id, value, attributes)

    @property
    def channel(self) -> str:
        """Get the channel of the State Repository."""
        return "multiple"

    async def async_read_states(self) -> None:
        """Read the states from the channel asynchronously."""
        for repository in self._repositories:
            await repository.async_read_states()

    def read_states(self) -> None:
        """Read the states from the channel."""
        for repository in self._repositories:
            repository.read_states()

    async def async_write_states(self) -> None:
        """Send the changed states to hass."""
        for repository in self._repositories:
            await repository.async_write_states()

    def write_states(self) -> None:
        """Write the states to the channel."""
        for repository in self._repositories:
            repository.write_states()


class PowerModes(StrEnum):
    """Power modes for controlling the device."""

    DEVICE_CONTROLLED = auto()
    OFF = auto()
    PV = auto()
    MIN_PV = auto()
    FAST = auto()
    OPTIMIZED = auto()


@dataclass
class Location:
    """Location of the home."""

    time_zone: str
    latitude: str
    longitude: str
    elevation: str

    def get_time_zone(self) -> tzinfo:
        """Get the timezone."""
        return ZoneInfo(self.time_zone)
